@@ -1,8 +1,9 @@
 import hashlib
 
-from flask import Flask, render_template
+from flask import Flask
 from flask import abort
 from flask import make_response
+from flask import render_template
 from flask import url_for
 
 from tiffslide.deepzoom import MinimalComputeAperioDZGenerator
@@ -15,7 +16,8 @@ _dzgen = None
 
 @app.route("/")
 def index():
-    fn = _dzgen._urlpath  # just so we don't need to worry about caching
+    # noinspection PyProtectedMember
+    fn = _dzgen._openfile.path  # just so we don't need to worry about caching
     su = url_for("dzi", image_fn=hashlib.sha1(fn.encode()).hexdigest())
     return render_template("index.html", filename=fn, slide_url=su)
 
