@@ -2,8 +2,8 @@ from __future__ import annotations
 
 import math
 import os.path
-import re
 import sys
+from collections import defaultdict
 from fractions import Fraction
 from itertools import count
 from types import TracebackType
@@ -12,35 +12,40 @@ from typing import Any
 from typing import AnyStr
 from typing import Iterator
 from typing import Mapping
+from typing import TypeVar
 from typing import overload
 from warnings import warn
 from xml.etree import ElementTree
 
 if sys.version_info[:2] >= (3, 8):
     from functools import cached_property
-    from importlib.metadata import version
     from typing import Literal
 else:
-    from backports.cached_property import cached_property
-    from importlib_metadata import version
+    from tiffslide._pycompat import cached_property
     from typing_extensions import Literal
 
+import numpy as np
 import tifffile
 import zarr
 from fsspec.core import url_to_fs
 from fsspec.implementations.local import LocalFileSystem
+from fsspec.implementations.reference import ReferenceFileSystem
 from PIL import Image
 from tifffile import TiffFile
 from tifffile import TiffFileError as TiffFileError
 from tifffile import TiffPageSeries
 from tifffile.tifffile import svs_description_metadata
 
+from tiffslide._compat import NotTiffFile
 from tiffslide._types import OpenFileLike
 from tiffslide._types import PathOrFileOrBufferLike
+from tiffslide._types import SeriesCompositionInfo
 from tiffslide._types import TiffFileIO
+from tiffslide._zarr import get_zarr_depth_and_dtype
+from tiffslide._zarr import get_zarr_selection
+from tiffslide._zarr import get_zarr_store
 
 if TYPE_CHECKING:
-    import numpy as np
     import numpy.typing as npt
 
 
@@ -58,11 +63,12 @@ __all__ = [
     "PROPERTY_NAME_BOUNDS_HEIGHT",
     "TiffSlide",
     "TiffFileError",
+    "NotTiffSlide",
 ]
 
 # all relevant tifffile version numbers work with this.
 _TIFFFILE_VERSION = tuple(
-    int(x) if x.isdigit() else x for x in version("tifffile").split(".")
+    int(x) if x.isdigit() else x for x in tifffile.__version__.split(".")
 )
 
 # === Constants to support drop-in ===
@@ -78,6 +84,12 @@ PROPERTY_NAME_BOUNDS_Y = "tiffslide.bounds-y"
 PROPERTY_NAME_BOUNDS_WIDTH = "tiffslide.bounds-width"
 PROPERTY_NAME_BOUNDS_HEIGHT = "tiffslide.bounds-height"
 
+# prevent pillow>=9.1.0 deprecation warning
+try:
+    _ANTIALIAS = Image.Resampling.LANCZOS
+except AttributeError:
+    _ANTIALIAS = Image.ANTIALIAS
+
 
 class TiffSlide:
     """
@@ -103,10 +115,27 @@ class TiffSlide:
             a dictionary with keyword arguments passed to fsspec
         """
         # tifffile instance, can raise TiffFileError
-        self.ts_tifffile = _prepare_tifffile(
-            filename, storage_options=storage_options, tifffile_options=tifffile_options
+        self._tifffile = _prepare_tifffile(
+            filename,
+            storage_options=storage_options,
+            tifffile_options=tifffile_options,
+            _cls=TiffFile,
         )
 
+    @property
+    def ts_tifffile(self) -> TiffFile:
+        """get the underlying tifffile instance"""
+        # backwards compatibility
+        if isinstance(self._tifffile, ReferenceFileSystem):
+            raise RuntimeError(
+                "instance is backed by kerchunk: no ts_tifffile available"
+            )
+        elif not isinstance(self._tifffile, (TiffFile, NotTiffFile)):
+            raise NotImplementedError(
+                f"instance backed by {type(self._tifffile).__name__}"
+            )
+        return self._tifffile
+
     def __enter__(self) -> TiffSlide:
         return self
 
@@ -119,12 +148,16 @@ class TiffSlide:
         self.close()
 
     def close(self) -> None:
-        ts_zarr_grp = self.__dict__.pop("ts_zarr_grp", None)
-        if ts_zarr_grp is not None:
+        try:
+            grp = self.__dict__.pop("zarr_group")
+        except KeyError:
+            pass
+        else:
             try:
-                self.ts_zarr_grp.close()
+                grp.close()
             except AttributeError:
-                pass  # Arrays don't need to be closed
+                pass
+            del grp
         self.ts_tifffile.close()
 
     def __repr__(self) -> str:
@@ -141,12 +174,15 @@ class TiffSlide:
         """return the detected format as a str or None if unknown/unimplemented"""
         try:
             tf = _prepare_tifffile(
-                filename, tifffile_options=tifffile_options, storage_options=storage_options
+                filename,
+                tifffile_options=tifffile_options,
+                storage_options=storage_options,
+                _cls=TiffFile,
             )
         except TiffFileError:
             return None
         with tf as t:
-            return _detect_format(t)
+            return _PropertyParser.detect_format(t)
 
     @cached_property
     def dimensions(self) -> tuple[int, int]:
@@ -192,103 +228,7 @@ class TiffSlide:
     @cached_property
     def properties(self) -> dict[str, Any]:
         """image properties / metadata as a dict"""
-        md: dict[str, Any] = {
-            PROPERTY_NAME_COMMENT: None,
-            PROPERTY_NAME_VENDOR: None,
-            PROPERTY_NAME_QUICKHASH1: None,
-            PROPERTY_NAME_BACKGROUND_COLOR: None,
-            PROPERTY_NAME_OBJECTIVE_POWER: None,
-            PROPERTY_NAME_MPP_X: None,
-            PROPERTY_NAME_MPP_Y: None,
-            PROPERTY_NAME_BOUNDS_X: None,
-            PROPERTY_NAME_BOUNDS_Y: None,
-            PROPERTY_NAME_BOUNDS_WIDTH: None,
-            PROPERTY_NAME_BOUNDS_HEIGHT: None,
-        }
-        tf = self.ts_tifffile
-
-        if tf.is_svs:
-            desc = tf.pages[0].description
-            series_idx = 0
-            _md = _parse_metadata_aperio(desc)
-
-        elif tf.is_scn:
-            desc = tf.scn_metadata
-            series_idx = _auto_select_series_scn(desc)
-            _md = _parse_metadata_scn(desc, series_idx)
-
-        else:
-            # todo: need to handle more supported formats in the future
-            if tf.is_bif or tf.is_ndpi:
-                vendor = _detect_format(tf)
-                warn(
-                    f"no special {vendor!r}-format metadata parsing implemented yet!"
-                )
-            desc = tf.pages[0].description
-            series_idx = 0
-            _md = {
-                PROPERTY_NAME_COMMENT: desc,
-                PROPERTY_NAME_VENDOR: "generic-tiff",
-            }
-
-        md.update(_md)
-        md["tiff.ImageDescription"] = desc
-        md["tiffslide.series-index"] = series_idx
-
-        # calculate level info
-        series0 = tf.series[series_idx]
-        assert series0.ndim == 3, "loosen restrictions in future versions"
-        axes0 = md["tiffslide.series-axes"] = series0.axes
-
-        if axes0 == "YXS":
-            h0, w0, _ = series0.shape
-            level_dimensions = ((lvl.shape[1], lvl.shape[0]) for lvl in series0.levels)
-        elif axes0 == "CYX":
-            _, h0, w0 = series0.shape
-            level_dimensions = ((lvl.shape[2], lvl.shape[1]) for lvl in series0.levels)
-        else:
-            raise NotImplementedError(f"series with axes={axes0!r} not supported yet")
-
-        for lvl, (width, height) in enumerate(level_dimensions):
-            downsample = math.sqrt((w0 * h0) / (width * height))
-            page = series0.levels[lvl].pages[0]
-            md[f"tiffslide.level[{lvl}].downsample"] = downsample
-            md[f"tiffslide.level[{lvl}].height"] = height
-            md[f"tiffslide.level[{lvl}].width"] = width
-            md[f"tiffslide.level[{lvl}].tile-height"] = page.tilelength
-            md[f"tiffslide.level[{lvl}].tile-width"] = page.tilewidth
-
-        if md[PROPERTY_NAME_MPP_X] is None or md[PROPERTY_NAME_MPP_Y] is None:
-            # recover mpp from tiff tags
-            try:
-                resolution_unit = series0.pages[0].tags["ResolutionUnit"].value
-                x_resolution = Fraction(*series0.pages[0].tags["XResolution"].value)
-                y_resolution = Fraction(*series0.pages[0].tags["YResolution"].value)
-            except KeyError:
-                pass
-            else:
-                md["tiff.ResolutionUnit"] = resolution_unit.name
-                md["tiff.XResolution"] = float(x_resolution)
-                md["tiff.YResolution"] = float(y_resolution)
-
-                RESUNIT = tifffile.TIFF.RESUNIT
-                scale = {
-                    RESUNIT.INCH: 25400.0,
-                    RESUNIT.CENTIMETER: 10000.0,
-                    RESUNIT.MILLIMETER: 1000.0,
-                    RESUNIT.MICROMETER: 1.0,
-                    RESUNIT.NONE: None,
-                }.get(resolution_unit, None)
-                if scale is not None:
-                    try:
-                        mpp_x = scale / x_resolution
-                        mpp_y = scale / y_resolution
-                    except ArithmeticError:
-                        pass
-                    else:
-                        md[PROPERTY_NAME_MPP_X] = mpp_x
-                        md[PROPERTY_NAME_MPP_Y] = mpp_y
-        return md
+        return _PropertyParser(self._tifffile).parse()
 
     @cached_property
     def associated_images(self) -> _LazyAssociatedImagesDict:
@@ -307,14 +247,28 @@ class TiffSlide:
         return self.level_count - 1
 
     @cached_property
-    def ts_zarr_grp(self) -> zarr.core.Array | zarr.hierarchy.Group:
-        """return the tiff image as a zarr array or group
+    def zarr_group(self) -> zarr.hierarchy.Group:
+        """return the tiff image as a zarr-like group
 
         NOTE: this is extra functionality and not part of the drop-in behaviour
         """
-        idx = self.properties["tiffslide.series-index"]
-        store = self.ts_tifffile.series[idx].aszarr()
-        return zarr.open(store, mode="r")
+        try:
+            _num_decode = os.environ["TIFFSLIDE_NUM_DECODE_THREADS"]
+        except KeyError:
+            num_decode_threads = None  # half of num CPU
+        else:
+            if _num_decode:
+                num_decode_threads = int(_num_decode)
+            else:
+                num_decode_threads = None
+        store = get_zarr_store(self.properties, self._tifffile, num_decode_threads=num_decode_threads)
+        return zarr.open_group(store, mode="r")
+
+    @property
+    def ts_zarr_grp(self) -> zarr.hierarchy.Group:
+        """use .zarr_group instead"""
+        # backwards compatibility
+        return self.zarr_group
 
     @overload
     def read_region(
@@ -333,6 +287,7 @@ class TiffSlide:
         size: tuple[int, int],
         *,
         as_array: Literal[False] = ...,
+        padding: bool = ...,
     ) -> Image.Image:
         ...
 
@@ -344,6 +299,7 @@ class TiffSlide:
         size: tuple[int, int],
         *,
         as_array: Literal[True] = ...,
+        padding: bool = ...,
     ) -> npt.NDArray[np.int_]:
         ...
 
@@ -354,6 +310,7 @@ class TiffSlide:
         size: tuple[int, int],
         *,
         as_array: bool = False,
+        padding: bool = True,
     ) -> Image.Image | npt.NDArray[np.int_]:
         """return the requested region as a PIL.Image
 
@@ -367,33 +324,82 @@ class TiffSlide:
             size (width, height) of the requested region
         as_array :
             if True, return the region as numpy array
+        padding :
+            if True, will ensure that the size of the returned image is deterministic.
         """
-        base_x, base_y = location
+        base_x, base_y = map(int, location)
         base_w, base_h = self.dimensions
-        level_w, level_h = self.level_dimensions[level]
+        _rw, _rh = map(int, size)
+        axes = self.properties["tiffslide.series-axes"]
+
+        try:
+            if level < 0:
+                raise IndexError
+            level_w, level_h = self.level_dimensions[level]
+        except IndexError:
+            if not padding:
+                raise IndexError(f"level={level} out of range")
+            warn(
+                f"level={level} is out-of-bounds, but padding is requested",
+                stacklevel=2,
+            )
+
+            depth, dtype = get_zarr_depth_and_dtype(self.zarr_group, axes)
+            return np.zeros((_rh, _rw, depth), dtype=dtype)
+
         rx0 = (base_x * level_w) // base_w
         ry0 = (base_y * level_h) // base_h
-        _rw, _rh = size
         rx1 = rx0 + _rw
         ry1 = ry0 + _rh
 
-        axes = self.properties["tiffslide.series-axes"]
+        in_bound = 0 <= rx0 and rx1 <= level_w and 0 <= ry0 and ry1 <= level_h
+        requires_padding = padding and not in_bound
+
+        if requires_padding:
+            # compute padding
+            pad_x0 = _clip(-rx0, 0, _rw)
+            pad_x1 = _clip(rx1 - level_w, 0, _rw)
+            pad_y0 = _clip(-ry0, 0, _rh)
+            pad_y1 = _clip(ry1 - level_h, 0, _rh)
+
+        if not in_bound:
+            # crop coord to valid zone
+            rx0 = _clip(rx0, 0, level_w)
+            rx1 = _clip(rx1, 0, level_w)
+            ry0 = _clip(ry0, 0, level_h)
+            ry1 = _clip(ry1, 0, level_h)
+
         if axes == "YXS":
             selection = slice(ry0, ry1), slice(rx0, rx1), slice(None)
         elif axes == "CYX":
             selection = slice(None), slice(ry0, ry1), slice(rx0, rx1)
         else:
-            raise NotImplementedError
+            raise NotImplementedError(f"axes={axes!r}")
 
-        arr: npt.NDArray[np.int_]
-        if isinstance(self.ts_zarr_grp, zarr.core.Array):
-            arr = self.ts_zarr_grp[selection]
-        else:
-            arr = self.ts_zarr_grp[str(level)][selection]
+        arr: npt.NDArray[np.int_] = get_zarr_selection(
+            self.zarr_group,
+            selection=selection,
+            level=level,
+        )
 
         if axes == "CYX":
             arr = arr.transpose((1, 2, 0))
 
+        if requires_padding:
+            if arr.shape[0] == 0 or arr.shape[1] == 0:
+                warn(
+                    f"location={location!r}, level={level}, size={size!r} is out-of-bounds, but padding is requested",
+                    stacklevel=2,
+                )
+
+            # noinspection PyUnboundLocalVariable
+            arr = np.pad(
+                arr,
+                ((pad_y0, pad_y1), (pad_x0, pad_x1), (0, 0)),
+                mode="constant",
+                constant_values=0,
+            )
+
         if as_array:
             return arr
         else:
@@ -446,22 +452,48 @@ class TiffSlide:
             color=f"#{self.properties[PROPERTY_NAME_BACKGROUND_COLOR] or 'ffffff'}",
         )
         thumb.paste(img, box=None, mask=None)
-        thumb.thumbnail(size, Image.ANTIALIAS)
+        thumb.thumbnail(size, _ANTIALIAS)
         return thumb
 
 
-def _detect_format(tf: TiffFile) -> str:
-    _vendor_compat_map = dict(
-        svs="aperio",
-        scn="leica",
-        bif="ventana",
-        ndpi="hamamatsu",
-        # add more when needed
-    )
-    for prop, vendor in _vendor_compat_map.items():
-        if getattr(tf, f"is_{prop}"):
-            return vendor
-    return "generic-tiff"
+class NotTiffSlide(TiffSlide):
+    # noinspection PyMissingConstructor
+    def __init__(
+        self,
+        filename: PathOrFileOrBufferLike[AnyStr],
+        *,
+        tifffile_options: dict[str, Any] | None = None,
+        storage_options: dict[str, Any] | None = None,
+    ) -> None:
+        # tifffile instance, can raise TiffFileError
+        self._tifffile = _prepare_tifffile(
+            filename,
+            storage_options=storage_options,
+            tifffile_options=tifffile_options,
+            _cls=NotTiffFile,
+        )
+
+    @classmethod
+    def detect_format(
+        cls,
+        filename: PathOrFileOrBufferLike[AnyStr],
+        *,
+        tifffile_options: dict[str, Any] | None = None,
+        storage_options: dict[str, Any] | None = None,
+    ) -> str | None:
+        """return the detected format as a str or None if unknown/unimplemented"""
+        try:
+            tf = _prepare_tifffile(
+                filename,
+                tifffile_options=tifffile_options,
+                storage_options=storage_options,
+                _cls=NotTiffFile,
+            )
+        except ValueError:
+            return None
+        with tf as t:
+            # noinspection PyProtectedMember
+            return t.pages[0]._codec
 
 
 class _LazyAssociatedImagesDict(Mapping[str, Image.Image]):
@@ -494,12 +526,16 @@ class _LazyAssociatedImagesDict(Mapping[str, Image.Image]):
         yield from self.series_map
 
 
+TF = TypeVar("TF", TiffFile, NotTiffFile)
+
+
 def _prepare_tifffile(
     fb: PathOrFileOrBufferLike[AnyStr],
     *,
     tifffile_options: dict[str, Any] | None = None,
     storage_options: dict[str, Any] | None = None,
-) -> TiffFile:
+    _cls: type[TF] = TiffFile,
+) -> TF:
     """prepare a TiffFile instance
 
     Allows providing fsspec urlpaths as well as fsspec OpenFile instances directly.
@@ -527,7 +563,7 @@ def _prepare_tifffile(
         # provided an IO stream like instance
         _warn_unused_storage_options(st_kw)
 
-        return TiffFile(fb, **tf_kw)
+        return _cls(fb, **tf_kw)
 
     elif isinstance(fb, OpenFileLike):
         # provided a fsspec compatible OpenFile instance
@@ -543,55 +579,221 @@ def _prepare_tifffile(
                 name = os.path.basename(path)
             tf_kw["name"] = name
 
-        return TiffFile(fs.open(path), **tf_kw)
+        return _cls(fs.open(path), **tf_kw)
 
     elif isinstance(fb, (str, os.PathLike)):
         # provided a string like url
         urlpath = os.fspath(fb)
         fs, path = url_to_fs(urlpath, **st_kw)
         if isinstance(fs, LocalFileSystem):
-            return TiffFile(path, **tf_kw)
+            return _cls(path, **tf_kw)
         else:
             # set name for tifffile.FileHandle
             if "name" not in tf_kw:
                 tf_kw["name"] = os.path.basename(path)
 
-            return TiffFile(fs.open(path), **tf_kw)
+            return _cls(fs.open(path), **tf_kw)
 
     else:
         # let's try anyways ...
         _warn_unused_storage_options(st_kw)
 
-        return TiffFile(fb, **tf_kw)
+        return _cls(fb, **tf_kw)
+
+
+# --- property / metadata related functionality -------------------------------
+
+
+class _PropertyParser:
+    """parse tiffslide properties for different slide types"""
+
+    vendor_map = dict(
+        svs="aperio",
+        scn="leica",
+        bif="ventana",
+        ndpi="hamamatsu",
+        # add more when needed
+    )
+
+    def __init__(self, tf: TiffFile) -> None:
+        self._tf = tf
+
+    @staticmethod
+    def new_metadata() -> dict[str, Any]:
+        return dict.fromkeys(
+            [
+                PROPERTY_NAME_COMMENT,
+                PROPERTY_NAME_VENDOR,
+                PROPERTY_NAME_QUICKHASH1,
+                PROPERTY_NAME_BACKGROUND_COLOR,
+                PROPERTY_NAME_OBJECTIVE_POWER,
+                PROPERTY_NAME_MPP_X,
+                PROPERTY_NAME_MPP_Y,
+                PROPERTY_NAME_BOUNDS_X,
+                PROPERTY_NAME_BOUNDS_Y,
+                PROPERTY_NAME_BOUNDS_WIDTH,
+                PROPERTY_NAME_BOUNDS_HEIGHT,
+            ]
+        )
+
+    @classmethod
+    def detect_format(cls, tf: TiffFile) -> str:
+        for prop, vendor in cls.vendor_map.items():
+            if getattr(tf, f"is_{prop}"):
+                return vendor
+        return "generic-tiff"
+
+    @classmethod
+    def collect_level_info(cls, series: TiffPageSeries) -> dict[str, Any]:
+        # calculate level info
+        md = {}
+        if series.ndim != 3:
+            raise NotImplementedError("currently no support for series.ndim != 3")
+
+        axes = md["tiffslide.series-axes"] = series.axes
+
+        if axes == "YXS":
+            h0, w0, _ = map(int, series.shape)
+            level_dimensions = ((lvl.shape[1], lvl.shape[0]) for lvl in series.levels)
+        elif axes == "CYX":
+            _, h0, w0 = map(int, series.shape)
+            level_dimensions = ((lvl.shape[2], lvl.shape[1]) for lvl in series.levels)
+        else:
+            raise NotImplementedError(f"series with axes={axes!r} not supported yet")
+
+        for lvl, (width, height) in enumerate(level_dimensions):
+            downsample = math.sqrt((w0 * h0) / (width * height))
+            page = series.levels[lvl][0]
+            md[f"tiffslide.level[{lvl}].downsample"] = downsample
+            md[f"tiffslide.level[{lvl}].height"] = int(height)
+            md[f"tiffslide.level[{lvl}].width"] = int(width)
+            md[f"tiffslide.level[{lvl}].tile-height"] = page.tilelength
+            md[f"tiffslide.level[{lvl}].tile-width"] = page.tilewidth
+        return md
+
+    @classmethod
+    def recover_mpp(cls, series: TiffPageSeries) -> dict[str, Any]:
+        """recover mpp from tiff tags"""
+        page0 = series[0]
+        md: dict[str, Any] = {}
+
+        try:
+            resolution_unit = page0.tags["ResolutionUnit"].value
+            x_resolution = Fraction(*page0.tags["XResolution"].value)
+            y_resolution = Fraction(*page0.tags["YResolution"].value)
+        except KeyError:
+            pass
+        else:
+            md["tiff.ResolutionUnit"] = resolution_unit.name
+            md["tiff.XResolution"] = float(x_resolution)
+            md["tiff.YResolution"] = float(y_resolution)
+
+            RESUNIT = tifffile.TIFF.RESUNIT
+            scale = {
+                RESUNIT.INCH: 25400.0,
+                RESUNIT.CENTIMETER: 10000.0,
+                RESUNIT.MILLIMETER: 1000.0,
+                RESUNIT.MICROMETER: 1.0,
+                RESUNIT.NONE: None,
+            }.get(resolution_unit, None)
+            if scale is not None:
+                try:
+                    mpp_x = scale / x_resolution
+                    mpp_y = scale / y_resolution
+                except ArithmeticError:
+                    pass
+                else:
+                    md[PROPERTY_NAME_MPP_X] = mpp_x
+                    md[PROPERTY_NAME_MPP_Y] = mpp_y
+        return md
+
+    def parse(self) -> dict[str, Any]:
+        fmt = self.detect_format(self._tf)
+        fmt = fmt.replace("-", "_")  # generic-tiff
+        return getattr(self, f"parse_{fmt}")()  # type: ignore
+
+    def parse_aperio(self) -> dict[str, Any]:
+        """parse Aperio SVS properties"""
+        md = self.new_metadata()
+
+        # parse metadata from description
+        desc = self._tf.pages[0].description
+        md.update(_parse_metadata_aperio(desc))
+        md["tiff.ImageDescription"] = desc
+
+        # get series 0
+        series0 = self._tf.series[0]
+        md["tiffslide.series-index"] = 0  # svs standard
+
+        # in case mpp wasn't available recover from tags
+        if not _has_mpp(md):
+            md.update(self.recover_mpp(series0))
+
+        # collect level info
+        md.update(self.collect_level_info(series0))
+        return md
+
+    def parse_leica(self) -> dict[str, Any]:
+        """parse leica SCN properties"""
+        md = self.new_metadata()
+
+        # parse metadata from scn xml
+        desc = self._tf.scn_metadata
+        md["tiff.ImageDescription"] = desc
+
+        # get all leica info
+        md.update(_parse_metadata_leica(desc))
+
+        # fill tile-width / tile-height
+        idx = md["tiffslide.series-index"]
+        for lvl, page in enumerate(self._tf.series[idx]):
+            md[f"tiffslide.level[{lvl}].tile-width"] = page.tilewidth
+            md[f"tiffslide.level[{lvl}].tile-height"] = page.tilelength
+
+        return md
+
+    def parse_ventana(self) -> dict[str, Any]:
+        warn(
+            "no special ventana-format metadata parsing implemented yet!",
+            stacklevel=2,
+        )
+        return self.parse_generic_tiff()
+
+    def parse_hamamatsu(self) -> dict[str, Any]:
+        warn(
+            "no special hamamatsu-format metadata parsing implemented yet!",
+            stacklevel=2,
+        )
+        return self.parse_generic_tiff()
+
+    def parse_generic_tiff(self) -> dict[str, Any]:
+        # todo: need to handle more supported formats in the future
+        md = self.new_metadata()
+
+        # store the description
+        desc = self._tf.pages[0].description
+        md["tiff.ImageDescription"] = desc
+
+        md["tiffslide.series-index"] = 0  # use series 0
+        series0 = self._tf.series[0]
+
+        # in case mpp wasn't available recover from tags
+        if not _has_mpp(md):
+            md.update(self.recover_mpp(series0))
+
+        # collect level info
+        md.update(self.collect_level_info(series0))
+        return md
 
 
 def _parse_metadata_aperio(desc: str) -> dict[str, Any]:
     """Aperio SVS metadata"""
-    if _TIFFFILE_VERSION >= (2021, 6, 14):
-        # tifffile 2021.6.14 fixed the svs parsing.
-        _aperio_desc = desc
-        _aperio_recovered_header = None  # no need to recover
-
-    else:
-        # this emulates the new description parsing for older versions
-        _aperio_desc = re.sub(r";Aperio [^;|]*(?=[|])", "", desc, 1)
-        _aperio_recovered_header = desc.split("|", 1)[0]
-
-    try:
-        aperio_meta = svs_description_metadata(_aperio_desc)
-    except ValueError:
-        raise
-    else:
-        # Normalize the aperio metadata
-        aperio_meta.pop("", None)
-        aperio_meta.pop("Aperio Image Library", None)
-        if aperio_meta and "Header" not in aperio_meta:
-            aperio_meta["Header"] = _aperio_recovered_header
-        vendor = "aperio"
+    aperio_meta = svs_description_metadata(desc)
+    assert "Header" in aperio_meta
 
     md = {
         PROPERTY_NAME_COMMENT: desc,
-        PROPERTY_NAME_VENDOR: vendor,
+        PROPERTY_NAME_VENDOR: "aperio",
         PROPERTY_NAME_QUICKHASH1: None,
         PROPERTY_NAME_BACKGROUND_COLOR: None,
         PROPERTY_NAME_OBJECTIVE_POWER: aperio_meta.get("AppMag", None),
@@ -606,73 +808,118 @@ def _parse_metadata_aperio(desc: str) -> dict[str, Any]:
     return md
 
 
-def _auto_select_series_scn(desc: str) -> int:
-    """selects the first non-macro series of an SCN file"""
-    tree = _xml_to_dict(desc)
+def _parse_metadata_leica(image_description: str) -> dict[str, Any]:
+    """return the leica SCN properties"""
+    # todo: clean up. this is pretty convoluted
+    md: dict[str, Any] = {PROPERTY_NAME_COMMENT: image_description}
 
-    marco_sizeX = int(tree["scn"]["collection"]["@sizeX"])
-    marco_sizeY = int(tree["scn"]["collection"]["@sizeY"])
+    dct = _xml_to_dict(image_description)
+    collection = dct["scn"]["collection"]
 
-    for idx, image in enumerate(tree["scn"]["collection"]["image"]):
-        offsetX = int(image["view"]["@offsetX"])
-        offsetY = int(image["view"]["@offsetY"])
-        sizeX = int(image["view"]["@sizeX"])
-        sizeY = int(image["view"]["@sizeY"])
+    slide_x_nm = int(collection["@sizeX"])
+    slide_y_nm = int(collection["@sizeY"])
+
+    first_non_macro_idx: int | None = None
+    lvl_resolutions = defaultdict(list)
+    series_offsets_nm = {}
+
+    for idx, image in enumerate(collection["image"]):
+        image_x_nm = int(image["view"]["@sizeX"])
+        image_y_nm = int(image["view"]["@sizeY"])
+        offset_x_nm = int(image["view"]["@offsetX"])
+        offset_y_nm = int(image["view"]["@offsetY"])
 
         is_macro_image = (
-            offsetX == 0
-            and offsetY == 0
-            and sizeX == marco_sizeX
-            and sizeY == marco_sizeY
+            offset_x_nm == 0
+            and offset_y_nm == 0
+            and image_x_nm == slide_x_nm
+            and image_y_nm == slide_y_nm
         )
-        if not is_macro_image:
-            break
-    else:
-        raise ValueError("SCN: no main image found")
+        if is_macro_image:
+            continue
 
-    return idx
+        if first_non_macro_idx is None:
+            first_non_macro_idx = idx
 
+            _scan = image["scanSettings"]
+            obj_pow = float(_scan["objectiveSettings"]["objective"])
+            aperture = float(_scan["illuminationSettings"]["numericalAperture"])
+            isource = str(_scan["illuminationSettings"]["illuminationSource"])
 
-def _parse_metadata_scn(desc: str, series_idx: int) -> dict[str, Any]:
-    """Leica metadata"""
-    tree = _xml_to_dict(desc)
+            md.update(
+                {
+                    PROPERTY_NAME_VENDOR: "leica",
+                    PROPERTY_NAME_OBJECTIVE_POWER: obj_pow,
+                    "leica.aperture": aperture,
+                    "leica.creation-date": str(image["creationDate"]),
+                    "leica.device-model": str(image["device"]["@model"]),
+                    "leica.device-version": str(image["device"]["@version"]),
+                    "leica.illumination-source": isource,
+                }
+            )
 
-    image = tree["scn"]["collection"]["image"][series_idx]
+        for lvl, info in enumerate(image["pixels"]["dimension"]):
+            resolution = image_x_nm / int(info["@sizeX"])
+            # image_y_nm / int(info["@sizeY"])  <-- openslide just uses X
+            lvl_resolutions[lvl].append(resolution)
 
-    # use auto recovery
-    # mpp_x = float(image['pixels']['@sizeX']) / float(image['view']['@sizeX']) / 1000
-    # mpp_y = float(image['pixels']['@sizeY']) / float(image['view']['@sizeY']) / 1000
-    mpp_x = None
-    mpp_y = None
+        series_offsets_nm[idx] = offset_y_nm, offset_x_nm
 
-    obj_pow = float(image["scanSettings"]["objectiveSettings"]["objective"])
+    if not lvl_resolutions:
+        raise ValueError("no non-macro images in file")
+    resolutions0 = np.array(lvl_resolutions[0])
+    # allow some threshold
+    _r_avg = resolutions0.mean()
+    if np.any(np.abs((resolutions0 - _r_avg) / _r_avg) > 0.02):
+        raise ValueError(
+            f"non-macro images vary too much in resolution: {lvl_resolutions[0]!r}"
+        )
 
-    md = {
-        PROPERTY_NAME_COMMENT: desc,
-        PROPERTY_NAME_VENDOR: "leica",
-        PROPERTY_NAME_QUICKHASH1: None,
-        PROPERTY_NAME_BACKGROUND_COLOR: None,
-        PROPERTY_NAME_OBJECTIVE_POWER: obj_pow,
-        PROPERTY_NAME_MPP_X: mpp_x,
-        PROPERTY_NAME_MPP_Y: mpp_y,
-        PROPERTY_NAME_BOUNDS_X: None,
-        PROPERTY_NAME_BOUNDS_Y: None,
-        PROPERTY_NAME_BOUNDS_WIDTH: None,
-        PROPERTY_NAME_BOUNDS_HEIGHT: None,
-        "leica.aperture": float(
-            image["scanSettings"]["illuminationSettings"]["numericalAperture"]
-        ),
-        "leica.creation-date": str(image["creationDate"]),
-        "leica.device-model": str(image["device"]["@model"]),
-        "leica.device-version": str(image["device"]["@version"]),
-        "leica.illumination-source": str(
-            image["scanSettings"]["illuminationSettings"]["illuminationSource"]
-        ),
-    }
+    nm_per_px = min(lvl_resolutions[0])
+    mpp = nm_per_px / 1000.0
+    md[PROPERTY_NAME_MPP_X] = mpp
+    md[PROPERTY_NAME_MPP_Y] = mpp
+
+    slide_x_px = math.ceil(slide_x_nm / nm_per_px)
+    slide_y_px = math.ceil(slide_y_nm / nm_per_px)
+
+    level_shapes = []
+    located_series = defaultdict(list)
+    for lvl, resolutions in sorted(lvl_resolutions.items()):
+        lvl_nm_per_px = min(resolutions)
+
+        for srs, offset_nm in series_offsets_nm.items():
+            # implicitly assuming axes="YXS" ... (might be wrong?)
+            offset_px = (
+                int(offset_nm[0] / lvl_nm_per_px),
+                int(offset_nm[1] / lvl_nm_per_px),
+                0,
+            )
+            located_series[srs].append(offset_px)
+        assert len(set(map(len, located_series.values()))) == 1
+
+        lvl_size_x = math.ceil(slide_x_nm / lvl_nm_per_px)
+        lvl_size_y = math.ceil(slide_y_nm / lvl_nm_per_px)
+        md[f"tiffslide.level[{lvl}].height"] = lvl_size_y
+        md[f"tiffslide.level[{lvl}].width"] = lvl_size_x
+        md[f"tiffslide.level[{lvl}].downsample"] = math.sqrt(
+            slide_x_px / lvl_size_x * slide_y_px / lvl_size_y
+        )
+        level_shapes.append((lvl_size_y, lvl_size_x, 3))
+
+    md["tiffslide.series-index"] = first_non_macro_idx
+    md["tiffslide.series-axes"] = "YXS"  # todo: verify
+    md["tiffslide.series-composition"] = SeriesCompositionInfo(
+        level_shapes=level_shapes,
+        located_series=located_series,
+    )
 
     return md
 
 
+# --- helper functions --------------------------------------------------------
+
+
 def _xml_to_dict(xml: str) -> dict[str, Any]:
     """helper function to convert xml string to a dictionary"""
     x = ElementTree.fromstring(xml)
@@ -701,3 +948,13 @@ def _xml_to_dict(xml: str) -> dict[str, Any]:
 def _label_series_axes(axes: str) -> tuple[str, ...]:
     """helper to make series shapes more understandable"""
     return tuple(tifffile.TIFF.AXES_LABELS[c] for c in axes)
+
+
+def _has_mpp(md: dict[str, Any]) -> bool:
+    """check if metadata has MPP defined"""
+    return md[PROPERTY_NAME_MPP_X] is not None and md[PROPERTY_NAME_MPP_Y] is not None
+
+
+def _clip(x: int, min_: int, max_: int) -> int:
+    """clip a value to a range"""
+    return min(max(x, min_), max_)
